package fr.gouv.vitamui.ui.commons.service;

import fr.gouv.vitamui.iam.common.dto.SubrogationDto;
import fr.gouv.vitamui.iam.common.enums.SubrogationStatusEnum;
import fr.gouv.vitamui.iam.external.client.SubrogationExternalRestClient;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentMatchers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.springframework.boot.test.context.TestConfiguration;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Import;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import java.time.OffsetDateTime;

import static org.assertj.core.api.Assertions.assertThat;

@RunWith(SpringJUnit4ClassRunner.class)
//@Import(SubrogationServiceTest.UserConfiguration.class)
public class SubrogationServiceTest extends ServiceTest<SubrogationDto> {

    @InjectMocks
    private SubrogationService service;

    @Mock
    private SubrogationExternalRestClient client;

    @Before
    public void setup() {
        Mockito.when(factory.getSubrogationExternalRestClient()).thenReturn(client);
        service = new SubrogationService(factory);
    }

    @Test
    public void testCreate() {
        super.createEntite();
    }

    @Test
    public void testUpdate() {
        super.updateEntite();
    }

    @Test
    public void testDelete() {
        super.delete();
    }

    @Test(expected = IllegalArgumentException.class)
    public void testUpdateWithIdEmpty() {
        super.updateWithIdEmpty();
    }

    @Test
    public void testAccept() {
        Mockito.when(client.accept(ArgumentMatchers.any(), ArgumentMatchers.any())).thenReturn(new SubrogationDto());
        final SubrogationDto subro = service.accept(null, "1");
        assertThat(subro).isNotNull();
    }

    @Test
    public void testGetMySubrogationAsSurrogate() {
        Mockito.when(client.getMySubrogationAsSurrogate(ArgumentMatchers.any())).thenReturn(new SubrogationDto());
        final SubrogationDto subro = service.getMySubrogationAsSurrogate(null);
        assertThat(subro).isNotNull();
    }

    @Test
    public void testGetMySubrogationAsSuperuser() {
        Mockito.when(client.getMySubrogationAsSuperuser(ArgumentMatchers.any())).thenReturn(new SubrogationDto());
        final SubrogationDto subro = service.getMySubrogationAsSuperuser(null);
        assertThat(subro).isNotNull();
    }

    @Test
    public void testDeline() {
        service.decline(null, "1");
    }

    @Override
    public SubrogationExternalRestClient getClient() {
        return client;
    }

    @Override
    protected SubrogationDto buildDto(final String id) {
        final SubrogationDto dto = new SubrogationDto();
        dto.setDate(OffsetDateTime.now());
        dto.setStatus(SubrogationStatusEnum.CREATED);
        dto.setSuperUser("superuser@vitamui.com");
        dto.setSurrogate("surrogate@vitamui.com");
        return dto;
    }

    @Override
    protected AbstractCrudService<SubrogationDto> getService() {
        return service;
    }

//    @TestConfiguration
//    static class UserConfiguration {
//
//        @Bean
//        public Validator validator() {
//            ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
//            return factory.getValidator();
//        }
//
//    }
}
